<?php

require_once 'base/FpatpcalcmovEntityBase.php';


/**
 * Enter description here ...
 *
 * @package Entity
 *
 * @author  alessandrobartels
 */
class FpatpcalcmovEntity extends FpaTpcalcmovEntityBase
{


    private $_nrs_calculo = array();


    /**
     * Retorna array com numeros de calculo.
     *
     * @return array
     */
    public function getNrcalculo() {
        return $this->_nrs_calculo;
    }


    /**
     * Enter description here ...
     *
     * @param integer $nr_calculo   Numero de Calculo.
     * @param integer $nr_tpcalcmov Numero TpCalcMov.
     * @param integer $indice       Indice.
     *
     * @return Ambiguous
     */
    public function setNrcalculo($nr_calculo, $nr_tpcalcmov, $indice) {
         $this->_nrs_calculo[$nr_tpcalcmov][$indice] = $nr_calculo;
         return $this->_nrs_calculo[$nr_tpcalcmov][$indice];
    }


    /**
     * Enter description here ...
     *
     * @param integer $nr_tipocalculo Numero tipo de calculo.
     *
     * @throws Exception Propaga Excecao.
     *
     * @return void
     */
    public function retornaTipoCalculoMovimento($nr_tipocalculo) {

        $resultado = Fpatpcalcmov::retornaTipoCalculoMovimento($nr_tipocalculo);

        if ($resultado !== null) {
            foreach ($resultado as $entity_obj) {
                $this->addItem($entity_obj, $entity_obj->getNrtpcalcmov());
            }
        } else {
            $message = 'FPATPCALCMOVENTITY  Nao foi encontrado movimentos para tipo de calculo ';
            $message .= $nr_tipocalculo.'<br/>';
            throw new Exception($message);
        }

    }


    /**
     * Enter description here ...
     *
     * @param integer $nr_def  Numero de definicao (nrtpcalcmov).
     * @param integer $nr_calc Numero de Calculo.
     *
     * @return FpatpcalcmovLogic
     */
    private function _adicionaNrCalculo($nr_def, $nr_calc) {

        $tp_calc_mov_logic = new FpatpcalcmovLogic($nr_calc, new FpaitecalcfolhaEntity());
        $definicoes =& $this->_nrs_calculo[$nr_def];
        $definicoes[] = $tp_calc_mov_logic;
        return $tp_calc_mov_logic;
    }


    /**
     * Atribui numero de calculo de folha. Faz tratamento de deslocamento da data de competencia.
     *
     * @param integer               $nrorg              Numero da Organizacao.
     * @param FpacalculofolhaEntity $calculo_folha      Objeto CalculoFolha.
     * @param TkDateTime            $dt_ocorrencia      Data de ocorrencia.
     * @param integer               $nrTpModalidadeCalc Numero do tipo de modalidade de calculo.
     * @param integer               $nrOcorrenciaCalc   Numero de ocorrencia de calculo.
     *
     * @return void
     *
     * @author alessandrobartels
     *         alessandrobartels - 20/12/2011 - adicionado parametros de tipo modalidade e ocorrencia de calculo.
     */
    public function atribuiNrCalculoFolha(
        $nrorg, FpacalculofolhaEntity $calculo_folha, TkDateTime $dt_ocorrencia, $nrTpModalidadeCalc, $nrOcorrenciaCalc
    ){
        $dt_deslocada = new TkDateTime(TkDateTime::DUMMY_DATE, TkDateTime::FORMAT_DATE);
        $itens = $this->getItems();

        foreach ($itens as $item) {
            /* @todo Verifica tipo do retorno (string) $item->getQtdeslocamento(). */
            if ((int)$item->getQtdeslocamento() !== 0) {
                $dt_deslocada->set($dt_ocorrencia);
                /* Se o deslocamento for negativo o addMonth funciona como subMonth. */
                $dt_deslocada->addMonth((int)$item->getQtdeslocamento());
            } else {
                $dt_deslocada->set($dt_ocorrencia);
            }

            /* $this->_atribuiNrCalculoFolha($calculo_folha, $nrorg, $item, $dt_deslocada); */
            $calc_folha = $calculo_folha->retornaCalculoFolha(
                $nrorg, $item->getNrtipomoviment(), $dt_deslocada, $nrTpModalidadeCalc, $nrOcorrenciaCalc
            );
            foreach ($calc_folha as $calc) {
                $this->_adicionaNrCalculo($item->getNrtpcalcmov(), $calc->getNrcalculofolha());
            }
        }
    }


    /**
     * Insere novo calculo de folha e retorna instancia de Fpatpcalcmov.
     * Este metodo eh usado pela preparacao de rescisao para criar novos numeros de calculo.
     * Quando o vinculo tem mais de uma ferias vencidas.
     *
     * @param FpacalculofolhaEntity $calculo_folha      Instancia de FpacalculofolhaEntity.
     * @param integer               $nrorg              Numero da Organizacao de Trabalho.
     * @param TkDateTime            $mesCompetencia     Mes de competencia.
     * @param integer               $nrTipoMovimento    Numero do Tipo de Movimento.
     * @param integer               $nrTpModalidadeCalc Numero do tipo de modalidade de calculo.
     * @param integer               $nrOcorrenciaCalc   Numero de ocorrencia de calculo.
     *
     * @return FpatpcalcmovEntity
     *
     * @author alessandrobartels 22/08/2011
     *         alessandrobartels 20/12/2011 - adicionado parametros de tipo modalidade e ocorrencia de calculo.
     */
    public function insereNrCalculoFolha(
        FpacalculofolhaEntity $calculo_folha, $nrorg, TkDateTime $mesCompetencia, $nrTipoMovimento, $nrTpModalidadeCalc,
        $nrOcorrenciaCalc
    ){

        /* alessandrobartels - 22/08/2011
         * Insere um novo calculo de folha em um movimento e data ja existentes.
         */
         $nrCalcFolha = $calculo_folha->inserePersisteCalculoFolha(
             $nrorg, $nrTipoMovimento, $nrTpModalidadeCalc, $nrOcorrenciaCalc, $mesCompetencia
         );

         /* Retorna a instancia FpatpcalcmovEntity. */
         $objTpCalcMov = $this->retornaTpCalcMovPorMovimento($nrTipoMovimento);

         /* Adiciona ao atributo (array) _nrs_calculo a instancia de FpatpcalcmovLogic.
          * Esta instancia contem um numero de calculo e a intancia de FpaitecalcfolhaEntity.
          */
         $this->_adicionaNrCalculo($objTpCalcMov->getNrtpcalcmov(), $nrCalcFolha);

        /* $this->_atribuiNrCalculoFolha($calculo_folha, $nrorg, $this, $mesCompetencia, true); */

        return $this;
    }


    /**
     * Retorna os itens de calculo.
     *
     * @param GpevinculomEntity $vinculo Instancia de GpevinculomEntity.
     *
     * @return boolean
     *
     * @author alessandrobartels
     */
    public function retornaItensCalculo(GpevinculomEntity $vinculo) {
        foreach ($this->_nrs_calculo as $nr_def => $itens) {
            $definicao = $this->_procuraTpCalcMov($nr_def);
            foreach ($itens as $posicao => $item) {
                $nr_calculo_folha = $item->getNrCalcFolha();
                $instancia = $item->getInstancia();
                $conta_itens = $instancia->retornaItensPorVinculo(
                    $vinculo->getNrorg(), $vinculo->getNrvinculom(), $nr_calculo_folha
                );
                if ((int)$conta_itens === 0) {
                    /* @todo tratar constantes. */
                    if ($definicao->getIdprincipal() === 'S') {
                        if ($definicao->getIdpodevazio() === 'N') {
                            return false;
                        }
                    }
                }
            }
        }

        return true;
    }


    /**
     * Persiste os itens de calculo.
     *
     * @return void
     *
     * @author alessandrobartels
     */
    public function persisteItensCalculo() {

        /* @todo gerar try/catch */
        foreach ($this->_nrs_calculo as $nr_def => $itens) {
            $definicao = $this->_procuraTpCalcMov($nr_def);
            foreach ($itens as $posicao => $item) {
                if ($definicao->getIdpersiste() === 'S') {
                    $instancia = $item->getInstancia();
                    $items = $instancia->getItems();
                    if (count($items) > 0) {
                        $instancia->persist();
                    }
                }
            }
        }
    }


    /**
     * Apaga da lista os itens calculo.
     *
     * @return void
     *
     * @author alessandrobartels
     */
    public function apagaItensCalculo(){

        /* @todo gerar try/catch */
        foreach ($this->_nrs_calculo as $nr_def => $itens) {
            foreach ($itens as $posicao => $item) {
                $instancia = $item->getInstancia();
                $apagou = $instancia->apagaItemCalculo();
                if ($apagou === true) {
                    $instancia->persist();
                }
            }
        }

    }


    /**
     * Enter description here ...
     *
     * @param TkDateTime       $mes_ocorrencia        Mes de ocorrencia.
     * @param FpaeventomEntity $evento                Instancia de FpaeventomEntity.
     * @param Elemento         $elementoDemonstrativo Instancia de Demonstrativo.
     * @param integer          $nrvinculom            Numero do vinculo.
     *
     * @return Ambiguous
     *
     * @author alessandrobartels
     */
    public function retornaListaItensCalculo(
        TkDateTime $mes_ocorrencia, FpaeventomEntity $evento, Elemento $elementoDemonstrativo = null,
        $nrvinculom = null
    ) {

        /* @todo gerar try/catch */
        foreach ($this->_nrs_calculo as $nr_def => $itens) {
            foreach ($itens as $posicao => $item) {
                $instancia = $item->getInstancia();
                $definicao = $this->_procuraTpCalcMov($nr_def);
                /* @todo Verificar tipo vindo como string. */

                if ((int)$definicao->getNrordimpreseven() !== 0) {
                    $itemDemonstrativo = $elementoDemonstrativo->addItem(
                        $definicao->getNmtpcalcmov(), $item->getNrCalcFolha(), $definicao->getNrordimpreseven(),
                        $nrvinculom
                    );

                    $itens = $instancia->retornaArrayItemCalculo($mes_ocorrencia, $evento, $itemDemonstrativo);

                    if (count($itens) > 0) {
                        $lista[] = $itens;
                    }

                } elseif ($definicao->getNrordimpreseven() === null) {
                    //@todo MsgCalculo
                    MsgCalculo::getInstance()->adicionaMensagem('Não foi definida a ordem de impressão no movimento.',MsgCalculo::ALERTA);
                    //tk_warning('FPATPCALCMOVENTITY - Nao foi definida a ordem de impressao no movimento.');
                }
            }
        }

        return $lista;
    }


    /**
     * Retorna a instancia de FpatpcalcmovLogic associada ao tipo de movimento parametrizado.
     * Procura no array nr_calculo.
     *
     * @param integer $nr_tp_mov    Numero tipo movimento.
     * @param integer $deslocamento Quantidade de deslocamento da data de competencia.
     *
     * @throws Exception Propaga Excecao.
     *
     * @return FpatpcalcmovLogic
     */
    public function retornaPosicaoTpMov($nr_tp_mov, $deslocamento = 0){
        foreach ($this->_nrs_calculo as $nr_def => $item) {
            $definicao = $this->_procuraTpCalcMov($nr_def);
            //if ($nr_tp_mov == 6) {
            //tk_warning($definicao->getNrtipomoviment(). ' -- '.$nr_tp_mov);
            //tk_warning($definicao->getQtdeslocamento(). ' ++ '.$deslocamento);
            //};
            if (((int)$definicao->getNrtipomoviment() === (int)$nr_tp_mov)
                &&((int)$definicao->getQtdeslocamento() === (int)$deslocamento)
            ) {
                return $item;
            }
        }
        //@todo MsgCalculo
        MsgCalculo::getInstance()->adicionaMensagem('Não foi encontrada associção do Tipo de Calculo com o Movimento '.$nr_tp_mov,MsgCalculo::ERRO,true);
        //tk_warning($nr_def.'  '.$definicao->getNrtipomoviment());
        $message = 'FPATPCALCMOVENTITY - Nao foi encontrada associacao do Tipo de Calculo com o Movimento '.$nr_tp_mov;
        $message .= ' com deslocamento '.$deslocamento.'<br/>';
        //tk_warning($message);
        //throw new Exception($message);
    }


    /**
     * Retorna a instancia de FpatpcalcmovEntity associada ao tipo de movimento parametrizado.
     * Procura na lista do Entity.
     *
     * @param integer $nrTipoMovimento Numero do Tipo de Movimento.
     *
     * @return FpatpcalcmovEntity|boolean
     *
     * @author alessandrobartels 22/08/2011
     */
    public function retornaTpCalcMovPorMovimento($nrTipoMovimento){
        $itens = $this->getItems();
        foreach ($itens as $item) {
            if ((int)$item->getNrtipomoviment() === (int)$nrTipoMovimento) {
                return $item;
            }
        }

        return false;
    }


    /**
     * Enter description here ...
     *
     * @param integer $nr_def Numero da definicao.
     *
     * @return FpatpcalcmovEntity
     */
    private function _procuraTpCalcMov($nr_def) {
        return $this->getItem($nr_def);
    }


    /**
     * Retorna o tipo de movimento  principal.
     *
     * @throws Exception Propagada Excecao.
     *
     * @return integer
     */
    public function retornaTpMovPrincipal() {
        foreach ($this->getItems() as $item) {
            if ($item->getIdprincipal() === 'S') {
                return $item->getNrtipomoviment();
            }
        }

        $mensagem = 'FPATPCALCMOVENTITY  Nao foi encontrado tipo de movimento principal<br/>';
        throw new Exception($mensagem);
    }


}

