IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
                    WHERE
	        O.NAME = 'RETIRADA_COZINHA_CENTRAL'  AND
	        TYPE = 'P'                        AND
	        U.UID = O.UID                     AND
	        U.NAME = USER
          )
  DROP PROCEDURE RETIRADA_COZINHA_CENTRAL
  GO

CREATE PROCEDURE RETIRADA_COZINHA_CENTRAL
   @P_CDFILIAL     VARCHAR(04),
   @P_CDSERVICO    VARCHAR(05),
   @P_DTRETIRADA   DATETIME,
   @P_DTCARDAPIO   DATETIME,
   @P_CDOPERADOR   VARCHAR(12),
   @P_CDFILIPROC   VARCHAR(04),
   @P_DSLANCAMENTO VARCHAR(60)
AS
   DECLARE @V_CDTIPORETI   VARCHAR(2);
   DECLARE @V_NRSEQUITRETI FLOAT;
   DECLARE @V_QTCONSPLAN   FLOAT;
   DECLARE @V_QTDEESTO     FLOAT;
   DECLARE @V_VALRESTO     FLOAT;
   DECLARE @V_VALRBRUT     FLOAT;
   DECLARE @V_PERCENT      FLOAT;
   DECLARE @V_NRCARDAPIO   VARCHAR(20);
   DECLARE @V_COUNT        FLOAT;
   DECLARE @V_CDFILIAL     VARCHAR(4);
   DECLARE @V_NRLANCTOEST  VARCHAR(20);
   DECLARE @V_NRRETIRADA   VARCHAR(20);
   DECLARE @V_QTCONSUMIDA  FLOAT;
   DECLARE @V_CDALMOXARIFE VARCHAR(6);
   DECLARE @V_CDLOCALESTOQ VARCHAR(6);
   DECLARE @V_IDUTILLOTE   VARCHAR(1);
   DECLARE @V_LOTEESTQ     VARCHAR(20);
   DECLARE @V_SUBLOTE      VARCHAR(4);

   DECLARE @V_NOVO_CODIGOAUX  VARCHAR(20);

   /*Cursor C_PRODRECEITA*/
   DECLARE @C_PROREC_DTRETIRADALANC DATETIME;
   DECLARE @C_PROREC_CDPRODUTO      VARCHAR(10);
   DECLARE @C_PROREC_VRUNITRECEPRAT FLOAT;
   DECLARE @C_PROREC_QTTOTARECEPRAT FLOAT;
   DECLARE @C_PROREC_IDRECEITAPRACA VARCHAR(1);
   DECLARE @C_PROREC_CDARVPROD      VARCHAR(20);
   DECLARE @C_PROREC_NMPRODUTO      VARCHAR(50);
   DECLARE @C_PROREC_SGUNIDADE      VARCHAR(10);
   DECLARE @C_PROREC_VRFATOCONV     FLOAT;
   DECLARE @C_PROREC_CDPRODESTO     VARCHAR(10);
   DECLARE @V_DTRETIRADALANCAUX     DATETIME;
BEGIN

   /*Tipo de Retirada default*/
   SELECT @V_CDTIPORETI = MAX(CDTPRETIPADR)
   FROM PARAMPLANEJA
   WHERE CDFILIAL = @P_CDFILIAL;

   SELECT @V_IDUTILLOTE = MAX(IDUTILLOTE)
   FROM PARAMFILIAL
   WHERE CDFILIAL = @P_CDFILIAL;

   SET @V_NRSEQUITRETI = 0;

   /*Verifica se existe cardpio para a data de retirada*/
   SELECT @V_NRCARDAPIO = MAX(NRCARDAPIO)
   FROM CARDADIA
   WHERE (CDFILIAL   = @P_CDFILIAL)
     AND (CDSERVFILI = @P_CDSERVICO)
     AND (DTCARDAPIO = @P_DTCARDAPIO);

   SET @V_COUNT = 0;

   /*Se existe cardpio, verifica se existe retirada correspondente*/
   IF @V_NRCARDAPIO IS NOT NULL BEGIN
       SELECT @V_COUNT = COUNT(1),
              @V_NRLANCTOEST = MAX(NRLANCESTQ),
              @V_NRRETIRADA = MAX(NRRETIRADA)
       FROM RETIRADA RE
       WHERE RE.CDFILIAL = @P_CDFILIAL
         AND RE.NRCARDAPIO = @V_NRCARDAPIO
         AND RE.DTRETIRADA = @P_DTCARDAPIO;
   END;
      
   /*Inclui cardpio e retirada caso no existam*/
   IF (@V_COUNT = 0) BEGIN
      IF @V_NRCARDAPIO IS NULL BEGIN
         SET @V_NOVO_CODIGOAUX = 'CARDAPIO' + @P_CDFILIAL;
         EXEC NOVO_CODIGO @V_NOVO_CODIGOAUX, @P_SEQUENCIAL = @V_NRCARDAPIO OUTPUT;
         SET @V_NRCARDAPIO = SUBSTRING(@V_NRCARDAPIO, 11, 10);

         INSERT INTO CARDADIA
            (CDFILIAL, NRCARDAPIO, CDSERVFILI, CDOPERADOR, DTCARDAPIO, DTMESCARD)
         VALUES
            (@P_CDFILIAL, @V_NRCARDAPIO, @P_CDSERVICO, @P_CDOPERADOR, @P_DTCARDAPIO, DATEADD(MM, -1, (DATEADD(MM,1, (@P_DTCARDAPIO)) - DAY(@P_DTCARDAPIO))) + 1);
      END;

      SET @V_NOVO_CODIGOAUX = 'LANCTOEST' + @P_CDFILIPROC;
      EXEC NOVO_CODIGO @V_NOVO_CODIGOAUX, @P_SEQUENCIAL = @V_NRLANCTOEST OUTPUT;
      SET @V_NRLANCTOEST = SUBSTRING(@V_NRLANCTOEST, 11, 10);

      INSERT INTO LANCTOESTOQ
        (CDFILIAL, NRLANCESTQ, IDTPLANCTO, DTLANCESTQ, DSLANCESTQ, IDALTCUSMED,
         NRLANCTONF, CDFILIMOVI, CDOPERADOR,  CDALMOXARIFE,  CDLOCALESTOQ)
      VALUES
        (@P_CDFILIAL, @V_NRLANCTOEST, '3', @P_DTCARDAPIO, @P_DSLANCAMENTO, 'N',
         NULL, @P_CDFILIAL, @P_CDOPERADOR,  ' ',  ' ');

      SET @V_NOVO_CODIGOAUX = 'RETIRADA' + @P_CDFILIAL;
      EXEC NOVO_CODIGO @V_NOVO_CODIGOAUX, @P_SEQUENCIAL = @V_NRRETIRADA OUTPUT;
      SET @V_NRRETIRADA = SUBSTRING(@V_NRRETIRADA, 11, 10);

      INSERT INTO RETIRADA
         (CDFILIAL, NRRETIRADA, NRCARDAPIO, DTRETIRADA, CDFILANCESTQ, NRLANCESTQ, IDRETIPRAT)
      VALUES
         (@P_CDFILIAL, @V_NRRETIRADA, @V_NRCARDAPIO, @P_DTRETIRADA, @P_CDFILIPROC, @V_NRLANCTOEST, 'N');

      SET @V_NRSEQUITRETI = 0;

   END
   ELSE BEGIN
      SELECT @V_NRSEQUITRETI = MAX(NRSEQUITRETI)
      FROM ITEMRETI
      WHERE CDFILIAL = @P_CDFILIAL
        AND NRRETIRADA = @V_NRRETIRADA
        AND NRCARDAPIO = @V_NRCARDAPIO;

      IF @V_NRSEQUITRETI IS NULL BEGIN
         SET @V_NRSEQUITRETI = 0;
      END;

      SET @V_NRSEQUITRETI = @V_NRSEQUITRETI + 1;
      
   END;

   DECLARE C_PRODRECEITA CURSOR FOR
      (SELECT RP.DTRETIRADALANC, RP.CDPRODUTO, RP.VRUNITRECEPRAT,
              SUM(RP.QTTOTARECEPRAT) AS QTTOTARECEPRAT, RP.IDRECEITAPRACA, PO.CDARVPROD,
              PO.NMPRODUTO, PO.SGUNIDADE, ISNULL(PO.VRFATOCONV, 1) AS VRFATOCONV,
              PO.CDPRODESTO
       FROM RECPRATPRODACAB RP, PRODUTO PO, PRODACABAPRATO PA
       WHERE RP.CDFILIAL = @P_CDFILIAL
         AND RP.DTRETIRADALANC = @P_DTRETIRADA
         AND PO.CDPRODUTO = RP.CDPRODUTO
         AND PA.CDFILIAL = RP.CDFILIAL
         AND PA.DTRETIRADALANC = RP.DTRETIRADALANC
         AND PA.CDPRODACABADO = RP.CDPRODACABADO
         AND PA.IDSTATUSPRODACA = 'C'
       GROUP BY RP.DTRETIRADALANC, RP.CDPRODUTO, RP.VRUNITRECEPRAT,
                RP.IDRECEITAPRACA, PO.CDARVPROD, PO.NMPRODUTO,
                PO.SGUNIDADE, ISNULL(PO.VRFATOCONV, 1),
                PO.CDPRODESTO)
   OPEN C_PRODRECEITA;

   FETCH NEXT FROM C_PRODRECEITA INTO @C_PROREC_DTRETIRADALANC, @C_PROREC_CDPRODUTO, @C_PROREC_VRUNITRECEPRAT,
                                      @C_PROREC_QTTOTARECEPRAT, @C_PROREC_IDRECEITAPRACA, @C_PROREC_CDARVPROD,
                                      @C_PROREC_NMPRODUTO, @C_PROREC_SGUNIDADE, @C_PROREC_VRFATOCONV, @C_PROREC_CDPRODESTO;
   WHILE (@@FETCH_STATUS = 0) BEGIN

      SET @V_DTRETIRADALANCAUX = @C_PROREC_DTRETIRADALANC - 1;
      EXEC CONSULTA_POSICAO @P_CDFILIAL,
	                    @C_PROREC_CDPRODUTO,
         	            @V_DTRETIRADALANCAUX,
                	    ' ',
                       	    ' ',
                            ' ',
                            ' ',
                            'N',
                            @V_QTDEESTO OUTPUT,
                            @V_VALRESTO OUTPUT,
                            @V_VALRBRUT OUTPUT;

      /*Calcula valor unitrio*/
      IF @V_QTDEESTO > 0 BEGIN
         SET @V_VALRBRUT = (@V_VALRBRUT / @V_QTDEESTO);
      END;

      IF (@V_QTDEESTO < @C_PROREC_QTTOTARECEPRAT) BEGIN
         SET @V_QTCONSUMIDA = ROUND((@C_PROREC_QTTOTARECEPRAT * @C_PROREC_VRFATOCONV), 5);
      END
      ELSE BEGIN
         SET @V_QTCONSUMIDA = ROUND((@C_PROREC_QTTOTARECEPRAT * @C_PROREC_VRFATOCONV), 5);
      END;

      IF @V_QTCONSUMIDA > 0 BEGIN

         EXEC LOCALESTOQUE_PADRAO @P_CDFILIPROC,
                                  @C_PROREC_CDPRODUTO,
                                  @V_CDALMOXARIFE,
                                  @V_CDLOCALESTOQ;

         IF @V_CDALMOXARIFE IS NULL BEGIN
            SET @V_CDALMOXARIFE = ' ';
         END;
         IF @V_CDLOCALESTOQ IS NULL BEGIN
            SET @V_CDLOCALESTOQ = ' ';
         END;

         EXEC CONSULTA_LOTE @V_IDUTILLOTE, @P_CDFILIPROC, @V_CDALMOXARIFE,
                            @V_CDLOCALESTOQ, @C_PROREC_CDPRODUTO,
                            @P_NRLOTEESTQ = @V_LOTEESTQ OUTPUT,
                            @P_NRSUBLOTE  = @V_SUBLOTE  OUTPUT;

         IF @V_LOTEESTQ IS NULL BEGIN 
            SET @V_LOTEESTQ = ' ';
         END;
         
         IF @V_SUBLOTE IS NULL BEGIN
            SET @V_SUBLOTE = ' ';
         END;

         INSERT INTO ITLANCTOEST
            (CDFILIAL, NRLANCESTQ, NRSEQUITEM, CDPRODUTO, QTLANCTOEST, VRUNILANCTO,
             VRTOTLANCTO, IDTIPOMOVI, DTLANCMOVI, IDORDEMCLC, QTTOTLANCTO,
             CDFILIMOVI, CDPRODMOVI, IDEVOLESTOQ, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE, CDTIPORETI)
         VALUES
            (@P_CDFILIPROC, @V_NRLANCTOEST, RIGHT(REPLICATE('0', 4) + @V_NRSEQUITRETI, 4), @C_PROREC_CDPRODUTO, @V_QTCONSUMIDA, @V_VALRBRUT,
             @C_PROREC_VRUNITRECEPRAT * @V_QTCONSUMIDA, '3', @P_DTCARDAPIO, '9', @V_QTCONSUMIDA,
             @P_CDFILIAL, @C_PROREC_CDPRODESTO, 'S', @V_CDALMOXARIFE, @V_CDLOCALESTOQ, @V_LOTEESTQ, @V_SUBLOTE, @V_CDTIPORETI);

         INSERT INTO ITEMRETI
            (CDFILIAL, NRRETIRADA, NRCARDAPIO, NRSEQUITRETI, CDPRODUTO, QTPREVISRETI, QTEXTRARETI, QTDEVOLVRETI, QTRETIRADA,
             CDFIITLANEST, NRLANCESTQ, NRSEQUITEM, CDTIPORETI)
         VALUES
            (@P_CDFILIAL, @V_NRRETIRADA, @V_NRCARDAPIO, RIGHT(REPLICATE('0', 4) + @V_NRSEQUITRETI, 4), @C_PROREC_CDPRODUTO, @V_QTCONSUMIDA, 0, 0, @V_QTCONSUMIDA,
             @P_CDFILIPROC, @V_NRLANCTOEST,  RIGHT(REPLICATE('0', 4) + @V_NRSEQUITRETI, 4), @V_CDTIPORETI);

      END
      ELSE BEGIN
         INSERT INTO ITEMRETI
            (CDFILIAL, NRRETIRADA, NRCARDAPIO, NRSEQUITRETI, CDPRODUTO, QTPREVISRETI, QTEXTRARETI, QTDEVOLVRETI, QTRETIRADA,
             CDFIITLANEST, NRLANCESTQ, NRSEQUITEM, CDTIPORETI)
         VALUES
            (@P_CDFILIAL, @V_NRRETIRADA, @V_NRCARDAPIO, (RIGHT(REPLICATE('0', 4) + @V_NRSEQUITRETI, 4)), @C_PROREC_CDPRODUTO, @V_QTCONSUMIDA, 0, 0, @V_QTCONSUMIDA,
             NULL, NULL, NULL, @V_CDTIPORETI);
      END;

      SET @V_NRSEQUITRETI = @V_NRSEQUITRETI + 1;

      FETCH NEXT FROM C_PRODRECEITA INTO @C_PROREC_DTRETIRADALANC, @C_PROREC_CDPRODUTO, @C_PROREC_VRUNITRECEPRAT,
                                         @C_PROREC_QTTOTARECEPRAT, @C_PROREC_IDRECEITAPRACA, @C_PROREC_CDARVPROD,
                                         @C_PROREC_NMPRODUTO, @C_PROREC_SGUNIDADE, @C_PROREC_VRFATOCONV, @C_PROREC_CDPRODESTO;

   END;
   CLOSE C_PRODRECEITA;
   DEALLOCATE C_PRODRECEITA;

   UPDATE GERAPRODACABADO
   SET NRCARDAPIO = @V_NRCARDAPIO,
       NRRETIRADA = @V_NRRETIRADA
   WHERE CDFILIAL = @P_CDFILIAL
     AND DTRETIRADALANC = @P_DTRETIRADA;  

END;
GO
